#include "ompi_config.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <stddef.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <getopt.h>
#include <time.h>
#include <ti/cmem.h>


#if OMPI_BTL_HAVE_SRIO
#include <linux/rio_mport_cdev.h>
#include <linux/rio_regs.h>
#endif

#include "opal/class/opal_bitmap.h"
#include "opal/datatype/opal_convertor.h"
#include "opal/sys/atomic.h"

#include "btl_srio.h"
#include "btl_srio_fifo.h"
#include "pktfwdK2H/mpi_interfaces.h"

/***************************************************************************************************/
extern void djdbg_update_queues(void);
extern void djdbg_report_queues(int tag_id);
extern int djdbg_last_outbound_port;
int srio_type11_fifo_write(int srio_dest_id, void *hdr1, int hdr1_len, void *hdr2, int hdr2_len, void *payload, int payload_len)
{
#if OMPI_BTL_HAVE_SRIO
    unsigned char *ptr, *base_ptr;
    void *cppi_desc;
    int complete_buff_len, buff_len, cnt;
    unsigned int seqn, max_seqn;
    unsigned char *loc_payload = (unsigned char *)payload;

    opal_output_verbose(20, mca_btl_base_output, "SRIO_FIFO_WRITE_TYPE11 payload %d headers: (%p)%d (%p)%d", 
                        payload_len, hdr1, hdr1_len, hdr2, hdr2_len);
    //btl_srio_dump_buffer("TXLOADBEGa", payload, 0, 4);
    //btl_srio_dump_buffer("TXLOADBEGb", payload, (payload_len - 1) / 4 - 4 * 8, 4);
    /* Transfer buffer to remote node */
    cnt = 0;
    do {
      /* Try to acquire free buffer */
//probe_pt_start(4);
      cppi_desc = Srio_Type11_AcquireTX (mca_btl_srio_component.type11_drv, &ptr, &buff_len, srio_dest_id);
//probe_pt_end(4);
      cnt ++;
      if(cnt > 10) {
        /* Ten consectuive attempts to acquire free buffer failed, sleep for 20 us */
        cnt = 0;
#if 0
        opal_output_verbose(0, mca_btl_base_output, "SRIO_FIFO_WRITE_TYPE11 A cannot acquire (srio_dest_id=%d, port=%d) (payload_len=%d)!", 
           srio_dest_id, djdbg_last_outbound_port, payload_len);
        djdbg_update_queues();
        djdbg_report_queues(1);
        usleep(1000000);
#else
        usleep(50);
#endif
      }
    } while(!cppi_desc);
    complete_buff_len = buff_len;
    seqn      = 0;
    buff_len -= 4 * sizeof(unsigned char); 
    max_seqn = (hdr1_len + hdr2_len + payload_len - 1) / buff_len; 
    *ptr++ = srio_dest_id;   /* destination ID */
    *ptr++ = mca_btl_srio_component.srio_id; /* source ID */
    *ptr++ = (unsigned char)seqn; /* 2K message sequence number */
    *ptr++ = (unsigned char)max_seqn; /* total number of 2K messages */
    base_ptr = ptr;
    memcpy (ptr, hdr1, hdr1_len);
    ptr   += hdr1_len;
    if(hdr2_len) {
      memcpy (ptr, hdr2, hdr2_len);
      ptr += hdr2_len;
    }
    buff_len -= (hdr1_len + hdr2_len);
    for (;;) {
      if(buff_len > payload_len) buff_len = payload_len;
//probe_pt_start(5);
      memcpy (ptr, loc_payload, buff_len);
      loc_payload += buff_len;
//      btl_srio_atomic_dsb(); /* << Memory barrier, make sure all the CPU transfers are done by this point, we make sure DDR is updated */
      btl_srio_atomic_dmb(); /* << Memory barrier, make sure all the CPU transfers are done by this point, we make sure DDR is updated */
      //CMEM_cacheWb(base_ptr - 4, 2048);
      /* Routing tag is already inserted into buffer, first location */
      opal_output_verbose(20, mca_btl_base_output, "SRIO_FIFO_WRITE_TYPE11 SEND: %p seqn %d max_seqn %d payload_len %d complete_buff_len %d buff_len %d dest_srio_id %d",
                          ptr, seqn, max_seqn, payload_len, complete_buff_len, buff_len, srio_dest_id);
      //btl_srio_dump_buffer("TXBEG", base_ptr, 0, 4);
      //btl_srio_dump_buffer("TXEND", base_ptr, 480, 4);
      Srio_Type11_SendTX (mca_btl_srio_component.type11_drv, cppi_desc, srio_dest_id, complete_buff_len);
      payload_len -= buff_len;
      seqn ++;
//probe_pt_end(5);
      if(payload_len <= 0) break;
      cnt = 0;
      do { 
        /* returns cppi_desc */
//probe_pt_start(6);
        cppi_desc = Srio_Type11_AcquireTX (mca_btl_srio_component.type11_drv, &ptr, &buff_len, srio_dest_id);
//probe_pt_end(6);
        cnt ++;
        if(cnt > 10) {
          cnt = 0;
#if 0
          opal_output_verbose(0, mca_btl_base_output, "SRIO_FIFO_WRITE_TYPE11 B cannot acquire (srio_dest_id=%d, port=%d) (payload_len=%d)!", 
            srio_dest_id, djdbg_last_outbound_port, payload_len);
          djdbg_update_queues();
          djdbg_report_queues(2);
          usleep(1000000);
#else
          usleep(50);
#endif
        }
      } while(!cppi_desc);
      opal_output_verbose(20, mca_btl_base_output, "SRIO_FIFO_WRITE_TYPE11 B acquired: %p buff_len %d dest_srio_id %d payload_leftover %d",
                          ptr, buff_len, srio_dest_id, payload_len);
      *ptr++ = (unsigned char)srio_dest_id;   /* destination ID */
      *ptr++ = (unsigned char)mca_btl_srio_component.srio_id; /* source ID */
      *ptr++ = (unsigned char)seqn; /* 2K message sequence number */
      *ptr++ = (unsigned char)max_seqn; /* total number of 2K messages */
      base_ptr = ptr;
      buff_len -= 4 * sizeof(unsigned char); 
    }
#endif
    opal_output_verbose(20, mca_btl_base_output, "SRIO_FIFO_WRITE_TYPE11 FINISHED to srio_id %d#", srio_dest_id);
    return 0;
}
/*** nothing past this point ***/
